
;**************************************************************************
;*                        1-wire Key Lock                                 *
;**************************************************************************
;**************************************************************************
;*                                                                        *
;*          v 1.11 - 26.01.2014                                           *
;*                                                                        *
;**************************************************************************
;*          Hardw.: PIC12F1840 used                                       *
;*          OSC.......: Internal osc. used     POWER.....:   5V extern.   *
;**************************************************************************


;Vss (Pin 8) -- GND
;RA0 (Pin 7) -- not used
;RA1 (Pin 6) -- I2C SCL
;RA2 (Pin 5) -- I2C SDA
;RA3 (Pin 4) -- function switch
;RA4 (Pin 3) -- Green LED
;RA5 (Pin 2) -- releas
;Vdd (Pin 1) -- +5V


 INCLUDE "P12F1840.INC"


_U001			     EQU     B'00100110000100'		;see Page 34 of Datasheet
_U002			     EQU     B'01100011111111'		;see Page 35 of Datasheet


	__CONFIG	_CONFIG1, _U001
	__CONFIG	_CONFIG2, _U002


;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"

v_tmp1		EQU	H'22'

GPIOA		EQU	H'23'
GPIOB		EQU	H'24'

i2cc_status	EQU	H'25'
	;bit 0 = 1 (error) if no ACK has been received
i2cb_status	EQU	H'26'
	;bit 0 = 1 (error) if no 1-wire bridge detected

ow_buffer	EQU	H'26'
ow_m1		EQU	H'27'		;temperature LSB
ow_m2		EQU	H'28'		;temperature MSB

ds2484_data	EQU	H'29'		;data buffer for 1-wire bridge

v_blink		EQU	H'2A'
v_bx		EQU	H'2B'
;-------------------------------
ee_buffer	EQU	H'2C'
ee_add		EQU	H'2D'
v_temp		EQU	H'2E'
;------------------------------
dev_id		EQU	H'2F'

rom_id0		EQU	H'30'		;LSB -- Family Code
rom_id1		EQU	H'31'		;serial number LSB
rom_id2		EQU	H'32'		;serial number
rom_id3		EQU	H'33'		;serial number
rom_id4		EQU	H'34'		;serial number
rom_id5		EQU	H'35'		;serial number
rom_id6		EQU	H'36'		;serial number MSB
rom_id7		EQU	H'37'		;MSB -- CRC
;-------------------------------

ds2484_sta	EQU	H'38'		;Status register of the bridge


ow_rom0		EQU	H'39'		;LSB -- Family Code
ow_rom1		EQU	H'3A'		;serial number LSB
ow_rom2		EQU	H'3B'		;serial number
ow_rom3		EQU	H'3C'		;serial number
ow_rom4		EQU	H'3D'		;serial number
ow_rom5		EQU	H'3E'		;serial number
ow_rom6		EQU	H'3F'		;serial number MSB
ow_rom7		EQU	H'40'		;MSB -- CRC
;-------------------------------
v_key_pos	EQU	H'41'
v_key_att	EQU	H'42'

;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;==========================================================================

		ORG	0		;Reset vector address
		goto	start

;==========================================================================
;--------------------------------------------------------------------------
;--------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************

start		nop
;
		clrf	INTCON		;disable interrupts

		movlw	B'10000000'
		movwf	OPTION_REG	;see datasheet Page 151

		movlw	H'03'
		movwf	BSR		;select BANK 3
		clrf	ANSELA

		movlw	H'01'
		movwf	BSR		;select BANK 1
		movlw	B'11001111'
		movwf	TRISA

		movf	OSCCON,0
		andlw	B'11101111'
		iorlw	B'01101000'
		movwf	OSCCON		;set 4MHz internal clock (default after reset is 500kHz)

		clrf	BSR		;select BANK 0

		clrf	PORTA
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;set up I2C BUS for communication as a master

		movlw	H'04'
		movwf	BSR		;select BANK 4

		movlw	B'00101000'		;see Datasheet Page 252
;			bit 3-0 SSPM<3:0>: Synchronous Serial Port Mode Select bits
;			1000 = I2C Master mode, clock = FOSC / (4 * (SSPADD+1))
		movwf	SSP1CON1

;set-up I2C speed
		movlw	B'00111111'
		movwf	SSP1ADD
		clrf	SSP1STAT

		clrf	BSR		;select BANK 0
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------

		clrf	PORTA
f_lb2		movlw	D'005'
		movwf	v_tmp1

f_lb		bsf	PORTA,D'004'
		call	d2
		bcf	PORTA,D'004'
		call	d2
		decfsz	v_tmp1,1
		goto	f_lb

		clrf	PORTA
;-----------------------------------------------------------------------------
		call	port_setup		;MCP23017 setup
		call	w_mm
		call	d2
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
main1		clrf	v_blink
		clrf	v_bx
		bcf	PORTA,D'004'

		btfss	PORTA,D'003'
		goto	main2		; --> normal mode
		goto	main3		; --> programming mode
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;   Standard mode -- entry
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
main2		nop
		bcf	PORTA,D'004'
;find out, if we have at least one "key" registered
		clrf	dev_id
		call	get_device_ee
;;;		call	show_device
;------------------------------------
		movf	rom_id0,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	main4		; --> standard mode, but no device registered
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;   Standard mode -- at least one device is registered
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
main5		nop

		call	check_key
		btfsc	v_key_att,D'007'
		goto	no_bridge
;-----------------------------------------------------------------------------
		btfsc	v_key_att,D'000'
		goto	main7			;registered key found
;-----------------------------------------------------------------------------
		btfsc	v_key_att,D'001'
		goto	m5_urk			;unregistered key attached
;-----------------------------------------------------------------------------
		clrf	GPIOA
		movlw	B'00111000'
		movwf	GPIOB
		call	LED_blink
		call	send_data

main5x		btfsc	PORTA,D'003'
		goto	main1		;change to programming mode
		goto	main5
;-----------------------------------------------------------------------------
;there was no response from 1-wire bridge detected
no_bridge	call	w_sE

		goto	main5x
;-----------------------------------------------------------------------------
;in standard mode unregistered key has been inserted
m5_urk		clrf	GPIOA
		movlw	B'00001001'
		movwf	GPIOB
		call	send_data

		goto	main5
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;   Standard Mode -- no device registered
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
main4		nop
		call	LED_blink
		call	w_mm
		btfsc	PORTA,D'003'
		goto	main1		;change to programming mode
		goto	main4
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;   Unlock mode -- registered device (key) has been recognized
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
main7		nop
		clrf	GPIOA
		movlw	B'00011100'
		movwf	GPIOB
		call	send_data
;
;-------- switch the releas ON ----------
		bsf	PORTA,D'005'
;----------------------------------------

		call	dr2
;show registration position
		call	w_ss
		call	d2
		movf	dev_id,0
		movwf	ow_m1
		incf	ow_m1,1		;position shown is 1 - 19 (starting with 1 not with 0)
		call	display_pos
		call	dr2
		call	w_ss
		call	d2

		call	check_key
		btfsc	v_key_att,D'000'
		goto	main7			;registered key is still there

;-------- switch the releas OFF ----------
		bcf	PORTA,D'005'
;-----------------------------------------

		goto	main5
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;   Programming mode
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
main3		nop
		bcf	PORTA,D'004'
;show "P" + RED Led
		movlw	B'10000000'
		movwf	GPIOA
		movlw	B'01110011'
		movwf	GPIOB
		call	send_data
;------------------------------------------
		call	check_key
		btfsc	v_key_att,D'000'
		goto	main3_1			;registered key inserted
		btfsc	v_key_att,D'001'
		goto	main3_2			;new key inserted
;------------------------------------------

		goto	main1
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; registered key has been inserted in programming mode
;      -- show the registration position and ROM ID of the key
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
main3_1		nop
		movf	dev_id,0
		movwf	ow_m1
		incf	ow_m1,1		;position shown is 1 - 19 (starting with 1 not with 0)
		call	display_pos
		call	dr2
		call	dr2
		call	dr2
		call	w_ss
		call	dr2
		call	show_1w_device		;show the ROM ID

		goto	main3
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; new key has been inserted in programming mode
;      -- register the key and return back
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
main3_2		nop
		movf	v_key_pos,0
		movwf	dev_id
		call	write_device_ee

		goto	main3
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
LED_blink	incf	v_blink,1
		movf	v_blink,0
		xorlw	D'020'
		btfsc	STATUS,Z
		call	blink_inv

		btfss	v_bx,D'000'
		goto	LED_1
		goto	LED_2

LED_1		bcf	PORTA,D'004'
		bsf	GPIOA,D'006'
		return

LED_2		bsf	PORTA,D'004'
		bcf	GPIOA,D'006'
		return

;-----------------------------------------------------------------------------
blink_inv	incf	v_bx,1
		clrf	v_blink
		return

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;*****************************************************************************
;*****************************************************************************
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;   CHECK KEY - check if there a device attached
;   result:
;       v_key_att - bit 0 = 1 if key is registered
;       v_key_att - bit 1 = 1 if key is unkown
;       v_key_att - bit 7 = 1 if there was no response from 1-wire bridge chip
;
;       v_key_pos - position of registered key (in EEPROM)
;       v_key_pos - position of first free position in EEPROM if key is not kown
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;chcek the bus first
check_key	clrf	v_key_att
		clrf	v_key_pos

		call	ow_rst
		call	ds2484_st_read
		movf	ds2484_sta,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	no_br

		btfss	ds2484_sta,D'001'		;PPD (Persence Pulse Detect)
		goto	no_key
;-------------------------------------------------------------------------------
;we found a device attached
		call	d2			;wait a while before reading ROM ID
		call	read_ROMid

		call	cmp_key

		return
;--------------------------------------------
no_key		nop				;nothing to do...
		return
;--------------------------------------------
no_br		bsf	v_key_att,D'007'
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
cmp_key		clrf	dev_id
;
cmp_key_loop	call	get_device_ee
		movf	rom_id0,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	cmp_key_f1		;no more devices registered
;
;compare entry from EEPROM with current values
		movf	rom_id0,0
		xorwf	ow_rom0,0
		btfss	STATUS,Z
		goto	cmp_key_f2

		movf	rom_id1,0
		xorwf	ow_rom1,0
		btfss	STATUS,Z
		goto	cmp_key_f2

		movf	rom_id2,0
		xorwf	ow_rom2,0
		btfss	STATUS,Z
		goto	cmp_key_f2

		movf	rom_id3,0
		xorwf	ow_rom3,0
		btfss	STATUS,Z
		goto	cmp_key_f2

		movf	rom_id4,0
		xorwf	ow_rom4,0
		btfss	STATUS,Z
		goto	cmp_key_f2

		movf	rom_id5,0
		xorwf	ow_rom5,0
		btfss	STATUS,Z
		goto	cmp_key_f2

		movf	rom_id6,0
		xorwf	ow_rom6,0
		btfss	STATUS,Z
		goto	cmp_key_f2

		movf	rom_id7,0
		xorwf	ow_rom7,0
		btfss	STATUS,Z
		goto	cmp_key_f2

;compare fits - we found registered key

		bsf	v_key_att,D'000'
		movf	dev_id,0
		movwf	v_key_pos		;Position of key in EEPROM

		return
;-----------------------------------------------------------------------------
;device not registered
cmp_key_f3	clrf	dev_id
cmp_key_f1	bsf	v_key_att,D'001'	;not found
		movf	dev_id,0
		movwf	v_key_pos		;free entry in EEPROM
		return
;-----------------------------------------------------------------------------
;compare missmatch
cmp_key_f2	incf	dev_id,1
		movf	dev_id,0
		xorlw	D'019'			;max 19 key can be registered (0 - 18)
		btfsc	STATUS,Z
		goto	cmp_key_f3

		goto	cmp_key_loop		;next one...
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

; Device 00 -- EEPROM address 00 - 07
; Device 01 -- EEPROM address 08 - 0F
; Device 02 -- EEPROM address 10 - 17
; Device 03 -- EEPROM address 18 - 1F
; .......

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;   read ROM ID of the device number "dev_id" from PIC EEPROM
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
get_device_ee	nop

		movf	dev_id,0
		movwf	ee_add
		rlf	ee_add,1
		rlf	ee_add,1
		rlf	ee_add,1
		bcf	ee_add,D'000'
		bcf	ee_add,D'001'
		bcf	ee_add,D'002'

		call	ee_read_byte
		movf	ee_buffer,0
		movwf	rom_id0

		incf	ee_add,1
		call	ee_read_byte
		movf	ee_buffer,0
		movwf	rom_id1

		incf	ee_add,1
		call	ee_read_byte
		movf	ee_buffer,0
		movwf	rom_id2

		incf	ee_add,1
		call	ee_read_byte
		movf	ee_buffer,0
		movwf	rom_id3

		incf	ee_add,1
		call	ee_read_byte
		movf	ee_buffer,0
		movwf	rom_id4

		incf	ee_add,1
		call	ee_read_byte
		movf	ee_buffer,0
		movwf	rom_id5

		incf	ee_add,1
		call	ee_read_byte
		movf	ee_buffer,0
		movwf	rom_id6

		incf	ee_add,1
		call	ee_read_byte
		movf	ee_buffer,0
		movwf	rom_id7

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
ee_read_byte	movf	ee_add,0
		banksel EEADRL
		movwf	EEADRL			;Data Memory
						;Address to read
		bcf	EECON1, CFGS		;Deselect Config space
		bcf	EECON1, EEPGD		;Point to DATA memory
		bsf	EECON1, RD		;EE Read
		movf	EEDATL, W		;W = EEDATL
		banksel	ee_buffer
		movwf	ee_buffer
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;   write ROM ID of the device number "dev_id" to PIC EEPROM
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
write_device_ee	nop

		movf	dev_id,0
		movwf	ee_add
		rlf	ee_add,1
		rlf	ee_add,1
		rlf	ee_add,1
		bcf	ee_add,D'000'
		bcf	ee_add,D'001'
		bcf	ee_add,D'002'

		movf	ow_rom0,0
		movwf	ee_buffer
		call	ee_write_byte

		incf	ee_add,1
		movf	ow_rom1,0
		movwf	ee_buffer
		call	ee_write_byte

		incf	ee_add,1
		movf	ow_rom2,0
		movwf	ee_buffer
		call	ee_write_byte

		incf	ee_add,1
		movf	ow_rom3,0
		movwf	ee_buffer
		call	ee_write_byte

		incf	ee_add,1
		movf	ow_rom4,0
		movwf	ee_buffer
		call	ee_write_byte

		incf	ee_add,1
		movf	ow_rom5,0
		movwf	ee_buffer
		call	ee_write_byte

		incf	ee_add,1
		movf	ow_rom6,0
		movwf	ee_buffer
		call	ee_write_byte

		incf	ee_add,1
		movf	ow_rom7,0
		movwf	ee_buffer
		call	ee_write_byte

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
ee_write_byte	movf	ee_add,0
		banksel EEADRL
		movwf	EEADRL			;Data Memory address
		banksel	ee_buffer
		movf	ee_buffer,0
		banksel EEADRL
		movwf	EEDATL

		bcf	EECON1, CFGS		;Deselect Config space
		bcf	EECON1, EEPGD		;Point to DATA memory
		bsf	EECON1, WREN		;EE Write Enable
		movf	EEDATL, W		;W = EEDATL
		bcf	INTCON, GIE		;Disable INTs. (just for sure here...)
		movlw	H'55'
		movwf	EECON2			;Write 55h
		movlw	H'AA'
		movwf	EECON2			;Write AAh
		bsf	EECON1, WR		;Set WR bit to begin write
		bcf	EECON1, WREN		;Disable writes
eewb2		btfsc	EECON1, WR		;Wait for write to complete
		goto	eewb2

		banksel	ee_buffer
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
show_device	movf	rom_id0,0
		movwf	ee_buffer
		call	show_byte
		call	dr2

		movf	rom_id1,0
		movwf	ee_buffer
		call	show_byte
		call	dr2

		movf	rom_id2,0
		movwf	ee_buffer
		call	show_byte
		call	dr2

		movf	rom_id3,0
		movwf	ee_buffer
		call	show_byte
		call	dr2

		movf	rom_id4,0
		movwf	ee_buffer
		call	show_byte
		call	dr2

		movf	rom_id5,0
		movwf	ee_buffer
		call	show_byte
		call	dr2

		movf	rom_id6,0
		movwf	ee_buffer
		call	show_byte
		call	dr2

		movf	rom_id7,0
		movwf	ee_buffer
		call	show_byte
		call	dr2

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
show_1w_device	movf	ow_rom0,0
		movwf	ee_buffer
		call	show_byte
		call	dr2

		movf	ow_rom1,0
		movwf	ee_buffer
		call	show_byte
		call	dr2

		movf	ow_rom2,0
		movwf	ee_buffer
		call	show_byte
		call	dr2

		movf	ow_rom3,0
		movwf	ee_buffer
		call	show_byte
		call	dr2

		movf	ow_rom4,0
		movwf	ee_buffer
		call	show_byte
		call	dr2

		movf	ow_rom5,0
		movwf	ee_buffer
		call	show_byte
		call	dr2

		movf	ow_rom6,0
		movwf	ee_buffer
		call	show_byte
		call	dr2

		movf	ow_rom7,0
		movwf	ee_buffer
		call	show_byte
		call	dr2

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
show_byte	nop

		movf	ee_buffer,0
		andlw	B'11110000'
		movwf	v_temp
		rrf	v_temp,1
		rrf	v_temp,1
		rrf	v_temp,1
		rrf	v_temp,1
		bcf	v_temp,D'007'
		bcf	v_temp,D'006'
		bcf	v_temp,D'005'
		bcf	v_temp,D'004'
		call	write_4b
		call	dr2
		call	w_ss
		call	d2

		movf	ee_buffer,0
		andlw	B'00001111'
		movwf	v_temp
		call	write_4b
		call	dr2
		call	w_ss
		call	d2

		return
;-----------------------------------------------------------------------------
write_4b	movf	v_temp,0
		xorlw	H'00'
		btfsc	STATUS,Z
		call	w_s0

		movf	v_temp,0
		xorlw	H'01'
		btfsc	STATUS,Z
		call	w_s1

		movf	v_temp,0
		xorlw	H'02'
		btfsc	STATUS,Z
		call	w_s2

		movf	v_temp,0
		xorlw	H'03'
		btfsc	STATUS,Z
		call	w_s3

		movf	v_temp,0
		xorlw	H'04'
		btfsc	STATUS,Z
		call	w_s4

		movf	v_temp,0
		xorlw	H'05'
		btfsc	STATUS,Z
		call	w_s5

		movf	v_temp,0
		xorlw	H'06'
		btfsc	STATUS,Z
		call	w_s6

		movf	v_temp,0
		xorlw	H'07'
		btfsc	STATUS,Z
		call	w_s7

		movf	v_temp,0
		xorlw	H'08'
		btfsc	STATUS,Z
		call	w_s8

		movf	v_temp,0
		xorlw	H'09'
		btfsc	STATUS,Z
		call	w_s9

		movf	v_temp,0
		xorlw	H'0A'
		btfsc	STATUS,Z
		call	w_sA

		movf	v_temp,0
		xorlw	H'0B'
		btfsc	STATUS,Z
		call	w_sB

		movf	v_temp,0
		xorlw	H'0c'
		btfsc	STATUS,Z
		call	w_sC

		movf	v_temp,0
		xorlw	H'0D'
		btfsc	STATUS,Z
		call	w_sD

		movf	v_temp,0
		xorlw	H'0E'
		btfsc	STATUS,Z
		call	w_sE

		movf	v_temp,0
		xorlw	H'0F'
		btfsc	STATUS,Z
		call	w_sF

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

display_pos	nop
;
		movf	ow_m1,0
		xorlw	D'000'
		btfsc	STATUS,Z
		goto	w_s0
;-------------------------------------
		movf	ow_m1,0
		xorlw	D'001'
		btfsc	STATUS,Z
		goto	w_s1
;-------------------------------------
		movf	ow_m1,0
		xorlw	D'002'
		btfsc	STATUS,Z
		goto	w_s2
;-------------------------------------
		movf	ow_m1,0
		xorlw	D'003'
		btfsc	STATUS,Z
		goto	w_s3
;-------------------------------------
		movf	ow_m1,0
		xorlw	D'004'
		btfsc	STATUS,Z
		goto	w_s4
;-------------------------------------
		movf	ow_m1,0
		xorlw	D'005'
		btfsc	STATUS,Z
		goto	w_s5
;-------------------------------------
		movf	ow_m1,0
		xorlw	D'006'
		btfsc	STATUS,Z
		goto	w_s6
;-------------------------------------
		movf	ow_m1,0
		xorlw	D'007'
		btfsc	STATUS,Z
		goto	w_s7
;-------------------------------------
		movf	ow_m1,0
		xorlw	D'008'
		btfsc	STATUS,Z
		goto	w_s8
;-------------------------------------
		movf	ow_m1,0
		xorlw	D'009'
		btfsc	STATUS,Z
		goto	w_s9
;-------------------------------------
		movf	ow_m1,0
		xorlw	D'010'
		btfsc	STATUS,Z
		goto	w_10
;-------------------------------------
		movf	ow_m1,0
		xorlw	D'011'
		btfsc	STATUS,Z
		goto	w_11
;-------------------------------------
		movf	ow_m1,0
		xorlw	D'012'
		btfsc	STATUS,Z
		goto	w_12
;-------------------------------------
		movf	ow_m1,0
		xorlw	D'013'
		btfsc	STATUS,Z
		goto	w_13
;-------------------------------------
		movf	ow_m1,0
		xorlw	D'014'
		btfsc	STATUS,Z
		goto	w_14
;-------------------------------------
		movf	ow_m1,0
		xorlw	D'015'
		btfsc	STATUS,Z
		goto	w_15
;-------------------------------------
		movf	ow_m1,0
		xorlw	D'016'
		btfsc	STATUS,Z
		goto	w_16
;-------------------------------------
		movf	ow_m1,0
		xorlw	D'017'
		btfsc	STATUS,Z
		goto	w_17
;-------------------------------------
		movf	ow_m1,0
		xorlw	D'018'
		btfsc	STATUS,Z
		goto	w_18
;-------------------------------------
		movf	ow_m1,0
		xorlw	D'019'
		btfsc	STATUS,Z
		goto	w_19
;-------------------------------------
		goto	w_zz

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;read temperature from DS18S20
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
ds1820_read_tmp	call	ow_rst

		movlw	H'CC'		;SKIP ROM
		movwf	ow_buffer
		call	ow_write

		movlw	H'BE'		;Read Scratchpad
		movwf	ow_buffer
		call	ow_write

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m1

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_m2

		return
;-------------------------------------------------------------------------------
ds1820_conv_spu	call	ow_rst
		movlw	H'CC'		;SKIP ROM
		movwf	ow_buffer
		call	ow_write

		movlw	H'44'		;Convert Temperature
		movwf	ow_buffer
		call	ow_write_spu

		call	d2		;wait a while...
		call	d2		;wait a while...
		call	d2		;wait a while...
		call	d2		;wait a while...
		call	d2		;wait a while...

		return
;-------------------------------------------------------------------------------

;###############################################################################
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;         1-wire communication routines utilizing bridge device DS2484
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;###############################################################################
;-------------------------------------------------------------------------------
;1-wire Reset
;-------------------------------------------------------------------------------
ow_rst		clrf	i2cb_status
		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'B4'			;command: 1-wire Reset
		call	i2c_send

		call	i2c_stop
		return
;-------------------------------------------------------------------------------
;1-wire write byte and use strong pull-up (SPU)
;                   byte to be written is in: ow_buffer (BANK 2)
;-------------------------------------------------------------------------------
ow_write_spu	nop

		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'D2'			;command: Write Device Configuration
		call	i2c_send
		movlw	B'10110100'		;activate SPU bit of cinfiguration register
		call	i2c_send
		call	i2c_stop

		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'A5'			;command: 1-wire Write Byte
		call	i2c_send
		movf	ow_buffer,0
		call	i2c_send		;byte to be written to 1-wire bus

		call	i2c_stop

		return
;-------------------------------------------------------------------------------
;1-wire write byte
;                   byte to be written is in: ow_buffer (BANK 2)
;-------------------------------------------------------------------------------
ow_write	nop

		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'A5'			;command: 1-wire Write Byte
		call	i2c_send
		movf	ow_buffer,0
		call	i2c_send		;byte to be written to 1-wire bus

		call	i2c_stop

		return
;-------------------------------------------------------------------------------
;1-wire read byte
;                   byte will be stored to: ow_buffer (BANK 2)
;-------------------------------------------------------------------------------
ow_read		nop

		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'96'			;command: 1-wire Read Byte
		call	i2c_send
		call	i2c_stop

		call	ds2484_da_read

		movf	ds2484_data,0
		movwf	ow_buffer
		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
read_ROMid	nop
ds1820_read_rom	call	ow_rst

		movlw	H'33'		;Read ROM
		movwf	ow_buffer
		call	ow_write

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_rom0

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_rom1

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_rom2

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_rom3

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_rom4

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_rom5

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_rom6

		call	ow_read
		movf	ow_buffer,0
		movwf	ow_rom7

		call	ow_rst
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
;###############################################################################
;###############################################################################
;###############################################################################
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ds2484_da_read	nop

		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'E1'			;set read pointer command
		call	i2c_send
		movlw	H'E1'			;address of the Read Data Register
		call	i2c_send
		call	i2c_restart		;repeat the start condition
		movlw	B'00110001'		;I2C Address of the brdige + read
		call	i2c_send

		call	i2c_receive
		movwf	ds2484_data
		call	i2c_not_ack
		call	i2c_stop

		return
;-------------------------------------------------------------------------------
ds2484_st_read	nop

		call	i2c_start
		movlw	B'00110000'		;I2C Address of the brdige + write
		call	i2c_send
		movlw	H'E1'			;set read pointer command
		call	i2c_send
		movlw	H'F0'			;address of the Status Register
		call	i2c_send
		call	i2c_restart		;repeat the start condition
		movlw	B'00110001'		;I2C Address of the brdige + read
		call	i2c_send

		call	i2c_receive
		movwf	ds2484_sta
		call	i2c_not_ack
		call	i2c_stop

		return
;-------------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
test01		call	w_s1
		call	d2

		call	w_s2
		call	d2

		call	w_s3
		call	d2

		call	w_s4
		call	d2

		call	w_s5
		call	d2

		call	w_s6
		call	d2

		call	w_s7
		call	d2

		call	w_s8
		call	d2

		call	w_s9
		call	d2

		call	w_10
		call	d2

		call	w_11
		call	d2

		call	w_12
		call	d2

		call	w_13
		call	d2

		call	w_14
		call	d2

		call	w_15
		call	d2

		call	w_16
		call	d2

		call	w_17
		call	d2

		call	w_18
		call	d2

		call	w_19
		call	d2

		clrf	GPIOA
		clrf	GPIOB
		bsf	GPIOA,D'005'
		call	send_data
		call	d2

		clrf	GPIOA
		clrf	GPIOB
		bsf	GPIOB,D'007'
		call	send_data
		call	d2

		clrf	GPIOA
		clrf	GPIOB
		bsf	GPIOA,D'002'		;minus
		call	send_data
		call	d2

		clrf	GPIOA
		clrf	GPIOB
		bsf	GPIOA,D'002'		;plus
		bsf	GPIOA,D'003'		;plus
		bsf	GPIOA,D'004'		;plus
		call	send_data
		call	d2


		clrf	GPIOA
		clrf	GPIOB
		bsf	GPIOA,D'006'
		call	send_data
		call	d2

		clrf	GPIOA
		clrf	GPIOB
		bsf	GPIOA,D'007'
		call	send_data
		call	d2

		call	w_2s			;swtich all off

		return
;-----------------------------------------------------------------------------
w_zz		movlw	B'00000100'		;minus
		movwf	GPIOA
		movlw	B'00000001'		;"minus hore"
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_mm		movlw	B'00000100'		;minus
		movwf	GPIOA
		movlw	B'01000000'		;minus
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_ss		movlw	B'00000000'		;space
		movwf	GPIOA
		movlw	B'00000000'		;space
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_su		clrf	GPIOA
		movlw	B'00011100'		;u
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_2s		clrf	GPIOA
		movlw	B'00000000'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_s1		clrf	GPIOA
		movlw	B'00000110'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_s2		clrf	GPIOA
		movlw	B'01011011'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_s3		clrf	GPIOA
		movlw	B'01001111'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_s4		clrf	GPIOA
		movlw	B'01100110'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_s5		clrf	GPIOA
		movlw	B'01101101'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_s6		clrf	GPIOA
		movlw	B'01111101'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_s7		clrf	GPIOA
		movlw	B'00000111'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_s8		clrf	GPIOA
		movlw	B'01111111'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_s9		clrf	GPIOA
		movlw	B'01101111'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_s0		clrf	GPIOA
		movlw	B'00111111'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_sA		clrf	GPIOA
		movlw	B'01110111'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_sB		clrf	GPIOA
		movlw	B'01111100'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_sC		clrf	GPIOA
		movlw	B'00111001'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_sD		clrf	GPIOA
		movlw	B'01011110'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_sE		clrf	GPIOA
		movlw	B'01111001'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_sF		clrf	GPIOA
		movlw	B'01110001'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_10		movlw	B'00000011'
		movwf	GPIOA
		movlw	B'00111111'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_11		movlw	B'00000011'
		movwf	GPIOA
		movlw	B'00000110'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_12		movlw	B'00000011'
		movwf	GPIOA
		movlw	B'01011011'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_13		movlw	B'00000011'
		movwf	GPIOA
		movlw	B'01001111'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_14		movlw	B'00000011'
		movwf	GPIOA
		movlw	B'01100110'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_15		movlw	B'00000011'
		movwf	GPIOA
		movlw	B'01101101'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_16		movlw	B'00000011'
		movwf	GPIOA
		movlw	B'01111101'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_17		movlw	B'00000011'
		movwf	GPIOA
		movlw	B'00000111'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_18		movlw	B'00000011'
		movwf	GPIOA
		movlw	B'01111111'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------
w_19		movlw	B'00000011'
		movwf	GPIOA
		movlw	B'01101111'
		movwf	GPIOB
		call	send_data
		return
;-----------------------------------------------------------------------------


;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;MCP23017 communication routines
;  we are using BANK = 0 and autoincrement feature (power on defaults)
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
port_setup	call	i2c_start
		movlw	B'01000000'		;MCP23017 I2C address (A2 = A1 = A0 = GND)
		call	i2c_send
		movlw	B'00000000'		;address of the IODIRA register
		call	i2c_send
		movlw	B'00000000'		;all bits of GPIOA to output
		call	i2c_send
		movlw	B'00000000'		;all bits to GPIOB output
		call	i2c_send

		call	i2c_stop
		return
;-----------------------------------------------------------------------------
send_data	call	i2c_start
		movlw	B'01000000'		;MCP23017 I2C address (A2 = A1 = A0 = GND)
		call	i2c_send
		movlw	H'14'			;GPIOA register
		call	i2c_send

		movf	GPIOA,0
		call	i2c_send

		movf	GPIOB,0
		call	i2c_send

		call	i2c_stop

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; I2C communication routines
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


;-----------------------------------------------------------------------------
i2c_start	bsf	BSR,D'002'		;select register BANK 4
		bsf	SSP1CON2, SEN
		bcf	BSR,D'002'		;select register BANK 0
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_stop	bsf	BSR,D'002'
		bsf	SSP1CON2, PEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_ack		bsf	BSR,D'002'
		bcf	SSP1CON2, ACKDT		;ACK
		bsf	SSP1CON2, ACKEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_not_ack	bsf	BSR,D'002'
		bsf	SSP1CON2, ACKDT		;NOT ACK
		bsf	SSP1CON2, ACKEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------
;output: received byte is in W register
;
i2c_receive	bsf	BSR,D'002'
		bsf	SSP1CON2, RCEN
		bcf	BSR,D'002'
		call	d55
		bsf	BSR,D'002'
		movf	SSP1BUF,0
		bcf	BSR,D'002'
		return
;-----------------------------------------------------------------------------
;input: transmitted byte have to be in W register
;
i2c_send	bsf	BSR,D'002'
		movwf	SSP1BUF
		bcf	BSR,D'002'
		call	d55			;wait a while
		return
;-----------------------------------------------------------------------------
i2c_restart	bsf	BSR,D'002'
		bsf	SSP1CON2, RSEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------






;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;---------------------------------------------------------------------------------------
;---------------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;		*********************************************
;		*               delay routine               *
;		*********************************************

;------------------------------------

d2		movlw	D'255'
		movwf	TIMER2
DEL_LOOP1	movlw	D'255'
		movwf	TIMER1

DEL_LOOP2	decfsz	TIMER1,F
		goto	DEL_LOOP2
		decfsz	TIMER2,F
		goto	DEL_LOOP1
		return
;------------------------------------
dr2		call	d2
		call	d2
		return
;------------------------------------
d1		movlw	D'003'
		movwf	TIMER2
DEL_LOOP11	movlw	D'255'
		movwf	TIMER1

DEL_LOOP12	decfsz	TIMER1,F
		goto	DEL_LOOP12
		decfsz	TIMER2,F
		goto	DEL_LOOP11
		return
;------------------------------------

d55		movlw	D'255'
		movwf	TIMER2
d55_loop	decfsz	TIMER2,F
		goto	d55_loop
		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

		END